@@ -22,10 +22,11 @@
 //	Enum to help provide descriptive names to array indices, when indexing into our various vectors
 enum clfftGenerators
 {
-	Stockham, // Using the Stockham autosort frameworks
-	Transpose,
-	Copy,
-	ENDGENERATORS			///< This value will always be last, and marks the length of clfftGenerators
+    Stockham, // Using the Stockham autosort frameworks
+    Transpose_VLIW,
+    Transpose_GCN,
+    Copy,
+    ENDGENERATORS			///< This value will always be last, and marks the length of clfftGenerators
 };
 
 #endif
