@@ -176,6 +176,7 @@ GpuStatTimer::Reset( )
 	if( nEvents == 0 || nSamples == 0 )
 		throw	std::runtime_error( "StatisticalTimer::Reserve( ) was not called before Reset( )" );
 
+	ReleaseEvents();
 	Reserve( nEvents, nSamples );
 
 	return;
@@ -203,9 +204,17 @@ void
 GpuStatTimer::AddSample( clfftPlanHandle plHandle, FFTPlan* plan, cl_kernel kern, cl_uint numEvents, cl_event* ev,
 	const std::vector< size_t >& gWorkSize )
 {
+	if( (numEvents != 0) && (ev == NULL) )
+		return;
+
 	if( timerData.empty( ) )
 		return;
 
+	for( size_t i = 0; i < numEvents; ++i )
+	{
+		::clRetainEvent(ev[i]);
+	}
+
 	if( currRecord == 0 )
 	{
 		timerData.at( currID ).push_back( StatDataVec( ) );
@@ -242,6 +251,26 @@ GpuStatTimer::getUniqueID( const std::string& label, cl_uint groupID )
 
 }
 
+void GpuStatTimer::ReleaseEvents()
+{
+	for( cl_uint id = 0; id < labelID.size( ); ++id )
+	{
+		for( size_t s = 0; s < timerData.at( id ).size( ); ++s )
+		{
+			for( size_t n = 0; n < timerData.at( id ).at( s ).size( ); ++n )
+			{
+				StatData& sd = timerData[ id ][ s ][ n ];
+
+				for( size_t i = 0; i < sd.outEvents.size( ); ++i )
+				{
+					::clReleaseEvent(sd.outEvents[ i ]);
+				}
+
+			}
+		}
+	}
+}
+
 void GpuStatTimer::queryOpenCL( size_t id )
 {
 	for( size_t s = 0; s < timerData.at( id ).size( ); ++s )
@@ -448,7 +477,9 @@ GpuStatTimer::Print( )
 						mean[ m ].plHandle == mean[ t ].planZ ||
 						mean[ m ].plHandle == mean[ t ].planTX ||
 						mean[ m ].plHandle == mean[ t ].planTY ||
-						mean[ m ].plHandle == mean[ t ].planTZ )
+						mean[ m ].plHandle == mean[ t ].planTZ ||
+						mean[ m ].plHandle == mean[ t ].planRCcopy ||
+						mean[ m ].plHandle == mean[ t ].planCopy )
 					{
 						time	+= mean[ m ].doubleNanoSec;
 					}
@@ -471,7 +502,8 @@ GpuStatTimer::Print( )
 			}
 
 			if( ( mean[ t ].planX + mean[ t ].planY + mean[ t ].planZ ) > 0 ||
-				( mean[ t ].planTX + mean[ t ].planTY + mean[ t ].planTZ ) > 0 )
+				( mean[ t ].planTX + mean[ t ].planTY + mean[ t ].planTZ ) > 0  || 
+				( mean[ t ].planRCcopy + mean[ t ].planCopy ) > 0 )
 			{
 				tout << std::setw( tableFourth ) << _T( "Child Handles:" );
 				catLengths.str( _T( "" ) );
@@ -503,6 +535,16 @@ GpuStatTimer::Print( )
 					catLengths << _T( "," );
 					catLengths << mean[ t ].planTZ;
 				}
+				if( mean[ t ].planRCcopy != 0 )
+				{
+					catLengths << _T( "," );
+					catLengths << mean[ t ].planRCcopy;
+				}
+				if( mean[ t ].planCopy != 0 )
+				{
+					catLengths << _T( "," );
+					catLengths << mean[ t ].planCopy;
+				}
 				catLengths << _T( ")" );
 				tout << std::setw( tableThird ) << catLengths.str( ) << std::endl;
 			}
