@@ -19,6 +19,7 @@
 #include <complex>
 #include "clFFT.h"
 #include "../client/openCL.misc.h"
+#include "test_constants.h"
 
 class clfft_UnitTest : public ::testing::Test {
 protected:
@@ -29,12 +30,12 @@ protected:
 		lengths[ 0 ] = 32;
 		lengths[ 1 ] = 32;
 		lengths[ 2 ] = 32;
-		cl_uint	deviceGpuList = ~0; // a bitmap set
+
 		commandQueueFlags = 0;
 
 		size_t memSizeBytes = lengths[ 0 ] * lengths[ 1 ] * lengths[ 2 ] * sizeof( std::complex< float > );
 
-		device_id = initializeCL( CL_DEVICE_TYPE_CPU, deviceGpuList, context, printInfo );
+		device_id = initializeCL( g_device_type, g_device_id, g_platform_id, context, printInfo );
 		createOpenCLCommandQueue( context,
 								  commandQueueFlags,
 								  queue,
